/*
Copyright © 2024 Mabel

All rights reserved. This plugin is protected by copyright law.

You may not modify, redistribute, or resell this software without explicit written permission from the copyright owner.

For any support plaese message me directly via Discord `mabel8686`
*/
using Newtonsoft.Json;
using Oxide.Core.Plugins;
using System;
using System.Collections.Generic;
using UnityEngine;
  
namespace Oxide.Plugins
{
    [Info("Backpack Wipe Block", "Mabel", "1.0.0")]
    [Description("Blocks use of backpacks for x amount of time after wipe")]

    public class BackpackWipeBlock : RustPlugin
    {
        [PluginReference] private Plugin Backpacks;

        private const string permission_bypass = "backpackwipeblock.bypass";
        static Configuration config;
        private DateTime wipeTime;

        public class BackpackSettings
        {
            [JsonProperty(PropertyName = "Block Duration Minutes")]
            public int BlockDurationMinutes;
        }

        public class Configuration
        {
            [JsonProperty(PropertyName = "Block Settings")]
            public BackpackSettings BackpackSettings { get; set; }

            public Core.VersionNumber Version { get; set; }

            public static Configuration DefaultConfig()
            {
                return new Configuration
                {
                    BackpackSettings = new BackpackSettings()
                    {
                        BlockDurationMinutes = 60
                    },
                    Version = new Core.VersionNumber(1,0,0)
                };
            }
        }

        protected override void LoadConfig()
        {
            base.LoadConfig();
            try
            {
                config = Config.ReadObject<Configuration>();
                if (config == null) LoadDefaultConfig();
                SaveConfig();

                if (config.Version < Version)
                    UpdateConfig();

                Config.WriteObject(config, true);
            }
            catch (Exception ex)
            {
                Debug.LogException(ex);
                PrintWarning("Creating New Configuration File.");
                LoadDefaultConfig();
            }
        }

        protected override void LoadDefaultConfig()
        {
            config = Configuration.DefaultConfig();
            Puts("Loaded Default Configuration File...");
        }

        protected override void SaveConfig()
        {
            Config.WriteObject(config);
        }

        private void UpdateConfig()
        {
            PrintWarning("Config Update Detected! Updating Configuration Values...");

            Configuration baseConfig = Configuration.DefaultConfig();
            if (config.Version < new Core.VersionNumber(1, 0, 0))
                config = baseConfig;

            config.Version = Version;
            PrintWarning("Configuration Update Completed!");
        }

        protected override void LoadDefaultMessages()
        {
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["Blocked"] = "Backpacks are blocked for another <color=green>{0}</color> days, <color=green>{1}</color> hours and <color=green>{2}</color> minutes after wipe!"
            }, this);
        }

        private void OnServerInitialized()
        {
            wipeTime = SaveRestore.SaveCreatedTime;
            permission.RegisterPermission(permission_bypass, this);
        }

        private bool IsBackpackBlocked()
        {
            TimeSpan timeSinceWipe = DateTime.UtcNow - wipeTime;
            return timeSinceWipe.TotalMinutes < config.BackpackSettings.BlockDurationMinutes;
        }

        private (int days, int hours, int minutes) GetRemainingBlockTime()
        {
            TimeSpan timeSinceWipe = DateTime.UtcNow - wipeTime;
            int remainingTotalMinutes = config.BackpackSettings.BlockDurationMinutes - (int)timeSinceWipe.TotalMinutes;

            int days = remainingTotalMinutes / (24 * 60);
            int remainingAfterDays = remainingTotalMinutes % (24 * 60);
            int hours = remainingAfterDays / 60;
            int minutes = remainingAfterDays % 60;

            return (days, hours, minutes);
        }

        string CanOpenBackpack(BasePlayer player, ulong backpackOwnerID)
        {
            if (permission.UserHasPermission(player.UserIDString, permission_bypass)) return null;

            if (IsBackpackBlocked())
            {
                var remainingTime = GetRemainingBlockTime();
                string message = string.Format(lang.GetMessage("Blocked", this, player.UserIDString), remainingTime.days, remainingTime.hours, remainingTime.minutes);
                player?.ShowToast(GameTip.Styles.Blue_Normal, message);
                return "";
            }
            return null;
        }
    }
}